/*  Data Analysis with Java
 *  John R. Hubbard
 *  June 6, 2017
 */

package dawj.ch08;

import java.util.HashSet;

public class HierarchicalClustering {
    private static final double[][] DATA = {{1,1}, {1,3}, {1,5}, {2,6}, {3,2}, 
        {3,4}, {4,3}, {5,6}, {6,3}, {6,4}, {7,1}, {7,5}, {7,6}};
    private static final int M = DATA.length;  // number of points
    private static final int K = 3;            // number of clusters

    public static void main(String[] args) {
        HashSet<Cluster> clusters = load(DATA);
        for (int i = 0; i < M - K; i++) {
            System.out.printf("%n%2d clusters:%n", M-i-1);
            coalesce(clusters);
            System.out.println(clusters);
        }
    }
    
    private static HashSet<Cluster> load(double[][] data) {
        HashSet<Cluster> clusters = new HashSet();
        for (double[] datum : DATA) {
            clusters.add(new Cluster(datum[0], datum[1]));
        }
        return clusters;
    } 
    
    private static void coalesce(HashSet<Cluster> clusters) {
        Cluster cluster1=null, cluster2=null;
        double minDist = Double.POSITIVE_INFINITY;
        for (Cluster c1 : clusters) {
            for (Cluster c2 : clusters) {
                if (!c1.equals(c2) && Cluster.distance(c1, c2) < minDist) {
                    cluster1 = c1;
                    cluster2 = c2;
                    minDist = Cluster.distance(c1, c2);
                }
            }
        }
        clusters.remove(cluster1);
        clusters.remove(cluster2);
        clusters.add(Cluster.union(cluster1, cluster2));
    }
}

/*
run:

12 clusters:
[
{(1.00,1.00),[(1.00,1.00)]}, 
{(1.00,3.00),[(1.00,3.00)]}, 
{(1.00,5.00),[(1.00,5.00)]}, 
{(2.00,6.00),[(2.00,6.00)]}, 
{(3.00,2.00),[(3.00,2.00)]}, 
{(3.00,4.00),[(3.00,4.00)]}, 
{(4.00,3.00),[(4.00,3.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(6.00,3.00),[(6.00,3.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(6.00,4.00),[(6.00,4.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

11 clusters:
[
{(1.00,1.00),[(1.00,1.00)]}, 
{(1.00,3.00),[(1.00,3.00)]}, 
{(1.00,5.00),[(1.00,5.00)]}, 
{(2.00,6.00),[(2.00,6.00)]}, 
{(3.00,2.00),[(3.00,2.00)]}, 
{(3.00,4.00),[(3.00,4.00)]}, 
{(4.00,3.00),[(4.00,3.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

10 clusters:
[
{(1.00,1.00),[(1.00,1.00)]}, 
{(1.00,3.00),[(1.00,3.00)]}, 
{(3.00,2.00),[(3.00,2.00)]}, 
{(3.00,4.00),[(3.00,4.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(4.00,3.00),[(4.00,3.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 9 clusters:
[
{(1.00,1.00),[(1.00,1.00)]}, 
{(1.00,3.00),[(1.00,3.00)]}, 
{(3.00,4.00),[(3.00,4.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(3.50,2.50),[(3.00,2.00), (4.00,3.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 8 clusters:
[
{(1.00,1.00),[(1.00,1.00)]}, 
{(1.00,3.00),[(1.00,3.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(3.33,3.00),[(3.00,4.00), (3.00,2.00), (4.00,3.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 7 clusters:
[
{(3.33,3.00),[(3.00,4.00), (3.00,2.00), (4.00,3.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(5.00,6.00),[(5.00,6.00)]}, 
{(1.00,2.00),[(1.00,1.00), (1.00,3.00)]}, 
{(7.00,5.50),[(7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 6 clusters:
[
{(3.33,3.00),[(3.00,4.00), (3.00,2.00), (4.00,3.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(6.00,3.50),[(6.00,3.00), (6.00,4.00)]}, 
{(1.00,2.00),[(1.00,1.00), (1.00,3.00)]}, 
{(6.33,5.67),[(5.00,6.00), (7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 5 clusters:
[
{(3.33,3.00),[(3.00,4.00), (3.00,2.00), (4.00,3.00)]}, 
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(6.20,4.80),[(6.00,3.00), (6.00,4.00), (5.00,6.00), (7.00,6.00), (7.00,5.00)]}, 
{(1.00,2.00),[(1.00,1.00), (1.00,3.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}]

 4 clusters:
[
{(1.50,5.50),[(1.00,5.00), (2.00,6.00)]}, 
{(6.20,4.80),[(6.00,3.00), (6.00,4.00), (5.00,6.00), (7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}, 
{(2.40,2.60),[(3.00,4.00), (3.00,2.00), (4.00,3.00), (1.00,1.00), (1.00,3.00)]}]

 3 clusters:
[
{(6.20,4.80),[(6.00,3.00), (6.00,4.00), (5.00,6.00), (7.00,6.00), (7.00,5.00)]}, 
{(7.00,1.00),[(7.00,1.00)]}, 
{(2.14,3.43),[(1.00,5.00), (2.00,6.00), (3.00,4.00), (3.00,2.00), (4.00,3.00), (1.00,1.00), (1.00,3.00)]}]
*/
